//
//  SubChannelListController.swift
//  SampleApp
//
//  Created by Amity on 28/10/2565 BE.
//  Copyright © 2565 BE David Zhang. All rights reserved.
//

import Foundation
import UIKit
import AmitySDK
import SwiftUI

class SubChannelListController: UIViewController, UITableViewDataSource, UITableViewDelegate {
    
    let tableView = UITableView()
    
    var channel: AmityChannel?
    let manager = SubChannelManager()
    
    var channelId: String = ""
    var fetchDeletedSubChannels = false
    var excludeMainSubChannel = false
    
    override func loadView() {
        view = UIView()
        view.backgroundColor = .white
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.title = "Sub Channels"
        
        tableView.translatesAutoresizingMaskIntoConstraints = false
        view.addSubview(tableView)
        
        NSLayoutConstraint.activate([
            tableView.leadingAnchor.constraint(equalTo: view.leadingAnchor),
            tableView.trailingAnchor.constraint(equalTo: view.trailingAnchor),
            tableView.topAnchor.constraint(equalTo: view.topAnchor),
            tableView.bottomAnchor.constraint(equalTo: view.bottomAnchor)
        ])
        
        tableView.register(UITableViewCell.self, forCellReuseIdentifier: UITableViewCell.identifier)
        tableView.dataSource = self
        tableView.delegate = self
        
        let moreOptionButton = UIBarButtonItem(image: UIImage(systemName: "ellipsis.circle"), style: .plain, target: self, action: #selector(onMoreOptionButtonTap))
        
        let filterButton = UIBarButtonItem(image: UIImage(systemName: "line.3.horizontal.decrease.circle"), style: .plain, target: self, action: #selector(onFilterButtonTap))
        self.navigationItem.rightBarButtonItems = [moreOptionButton, filterButton]
        
        guard let channelId = channel?.channelId else { return }
        self.channelId = channelId
        
        loadSubChannels()
    }
    
    func loadSubChannels() {
        manager.fetchSubChannels(channelId: channelId, isDeleted: fetchDeletedSubChannels, excludeMainChannel: excludeMainSubChannel) { [weak self] subChannels, error in
            
            // Reload tableview
            self?.tableView.reloadData()
        }
    }
    
    @objc func onMoreOptionButtonTap() {
        let createAction = UIAlertAction(title: "Create SubChannel", style: .default) { [weak self] action in
            let hostingController = UIHostingController(rootView: CreateSubChannelView(channelId: self?.channelId ?? ""))
            hostingController.title = "Create"
            self?.navigationController?.pushViewController(hostingController, animated: true)
        }
        
        let singleQueryAction = UIAlertAction(title: "Get SubChannel", style: .default) { [weak self] action in
            let hostingController = UIHostingController(rootView: SingleSubChannelQueryView())
            self?.navigationController?.pushViewController(hostingController, animated: true)
        }

        AppUtility.showActionSheet(in: self, title: "", message: "SubChannels", actions: [createAction, singleQueryAction])
    }
    
    @objc func onAddButtonTap() {
        let hostingController = UIHostingController(rootView: CreateSubChannelView(channelId: self.channelId))
        hostingController.title = "Create"
        self.navigationController?.pushViewController(hostingController, animated: true)
    }
    
    @objc func onFilterButtonTap() {
        AppUtility.showActionSheet(in: self, title: "Filter SubChannels", message: "Filter SubChannels By:", actions: [
            UIAlertAction(title: "IsDeleted: True", style: .default) { [weak self] action in
                self?.fetchDeletedSubChannels = true
                self?.loadSubChannels()
            },
            
            UIAlertAction(title: "IsDeleted: False", style: .default) { [weak self] action in
                self?.fetchDeletedSubChannels = false
                self?.loadSubChannels()
            },
            
            UIAlertAction(title: "Exclude Main Channel", style: .default) { [weak self] action in
                self?.excludeMainSubChannel = true
                self?.loadSubChannels()
            },
            
            UIAlertAction(title: "Include Main Channel", style: .default) { [weak self] action in
                self?.excludeMainSubChannel = false
                self?.loadSubChannels()
            }
        ])
    }
    
    func subscribeToSubChanenl(subChannel: AmitySubChannel) {
        subChannel.subscribeEvent { [weak self] isSuccess, error in
            guard let strongSelf = self else { return }
            
            let message = isSuccess ? "Subscribed Successfully: \(subChannel.displayName)" : "Failed to subscribe event"
            AppUtility.showAlert(in: strongSelf, title: "", message: message, action: nil)
        }
    }
    
    func unSubscribeSubChannel(subChannel: AmitySubChannel) {
        subChannel.unSubscribeEvent { [weak self] isSuccess, error in
            guard let strongSelf = self else { return }
            
            let message = isSuccess ? "Subscribed Successfully: \(subChannel.displayName)" : "Failed to subscribe event"
            AppUtility.showAlert(in: strongSelf, title: "", message: message, action: nil)
        }
    }
}

extension SubChannelListController {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return manager.subChannels.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: UITableViewCell.identifier)
        
        let item = manager.subChannels[indexPath.row]
        cell?.textLabel?.text = item.displayName
        
        let details = """
        Id: \(item.id),
        Channel Id: \(item.channelId),
        Is Deleted: \(item.isDeleted),
        Creator Id: \(item.creatorId)
        """
        
        cell?.detailTextLabel?.text = details
        return cell!
    }
    
    func scrollViewWillEndDragging(_ scrollView: UIScrollView, withVelocity velocity: CGPoint, targetContentOffset: UnsafeMutablePointer<CGPoint>) {
        let frameHeight = scrollView.frame.size.height
        let contentOffset = scrollView.contentOffset.y
        let contentHeight = scrollView.contentSize.height
        
        let distanceFromBottom = contentHeight - contentOffset
        if distanceFromBottom < frameHeight {
            manager.fetchMoreSubChannels()
        }
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let subChannel = manager.subChannels[indexPath.row]
        // If its deleted subchannel, do not allow user to do anything
        guard !subChannel.isDeleted else { return }
        
        let editAction = UIAlertAction(title: "Edit", style: .default) { [weak self] action in
            self?.showEditScreen(subChannel: subChannel)
        }
        
        let chatAction = UIAlertAction(title: "Chat Screen", style: .default) { [weak self] action in
            self?.displayChatScreen(subChannelId: subChannel.id)
        }
        
        let subscribeAction = UIAlertAction(title: "Subscribe Events", style: .default) { [weak self] action in
            self?.subscribeToSubChanenl(subChannel: subChannel.object)
        }
        
        let unSubscribeAction = UIAlertAction(title: "Un Subscribe Events", style: .default) { [weak self] action in
            self?.unSubscribeSubChannel(subChannel: subChannel.object)
        }
        
        AppUtility.showActionSheet(in: self, title: "", message: "Options", actions: [chatAction, editAction, subscribeAction, unSubscribeAction])
    }
    
    func showEditScreen(subChannel: AmitySubChannel.ViewModel) {
        let editView = EditSubChannelView(viewModel: self.manager, subChannelId: subChannel.id, subChannelName: subChannel.displayName)
        let editHostingController = UIHostingController(rootView: editView)
        editHostingController.navigationController?.title = "Edit SubChannel"
        
        self.navigationController?.pushViewController(editHostingController, animated: true)
    }
    
    func displayChatScreen(subChannelId: String) {
        let storyboard = UIStoryboard(name: "Chats", bundle: nil)
        let viewController = storyboard.instantiateViewController(withIdentifier: "AmityChat")
        guard let chatViewController = viewController as? AmityChatViewController, let channel = self.channel else { return }
        
        chatViewController.channelId = channelId
        chatViewController.client = AmityManager.shared.client!
        chatViewController.subChannelId = subChannelId
        chatViewController.channelType = channel.channelType
        
        self.navigationController?.pushViewController(chatViewController, animated: true)
    }
}
