//
//  CommentCreateTestView.swift
//  SampleApp
//
//  Created by Nishan Niraula on 2/24/21.
//  Copyright © 2021 David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

struct CommentCreateTestView: View {
    
    @State var referenceId: String = ""
    @State var commentText: String = ""
    @State var selectedReferenceType: String = ""
    
    @ObservedObject var viewModel = CreateCommentTestViewModel()
    
    var referenceTypes: [String] = [AmityCommentReferenceType.post.identifier, AmityCommentReferenceType.content.identifier]
    
    var body: some View {
        Form {
            Section(header: Text("Create Comment")) {
                TextField("Enter Post/Content Id", text: $referenceId)
                TextField("Enter comment", text: $commentText)
                Picker("Reference Type", selection: $selectedReferenceType) {
                    ForEach(referenceTypes, id: \.self) { item in
                        Text(item)
                    }
                }
            }
            
            Section {
                Button(action: {
                    viewModel.createComment(contentId: referenceId, comment: commentText, referenceType: selectedReferenceType)
                }, label: {
                    HStack {
                        Spacer()
                        Text("Create")
                        Spacer()
                    }
                })
            }
            
            Section(header: Text("Here you will see local & server result received from sdk after creation")) {
                VStack(alignment: .leading) {
                    Text("Local Result")
                        .font(.system(size: 18))
                        .fontWeight(.bold)
                    Text(viewModel.localOutput)
                }
                
                VStack(alignment: .leading) {
                    Text("Server Result")
                        .font(.system(size: 18))
                        .fontWeight(.bold)
                    Text(viewModel.serverOutput)
                }
            }
        }
        .navigationBarTitle("Comment Test", displayMode: .inline)
    }
}

struct CommentCreateTestView_Previews: PreviewProvider {
    static var previews: some View {
        CommentCreateTestView()
    }
}

class CreateCommentTestViewModel: ObservableObject {
    
    var commentRepo: AmityCommentRepository = AmityCommentRepository(client: AmityManager.shared.client!)
    var token: AmityNotificationToken?
    
    @Published var localOutput = ""
    @Published var serverOutput = ""
    
    func createComment(contentId:String, comment: String, referenceType: String) {
        let actualType: AmityCommentReferenceType = AmityCommentReferenceType.post.identifier == referenceType ? .post : .content
             
        let createOptions = AmityCommentCreateOptions(referenceId: contentId, referenceType: actualType, text: comment, parentId: nil, metadata: nil, mentioneeBuilder: nil)
        Task { @MainActor in
            do {
                let comment = try await commentRepo.createComment(with: createOptions)
                let commentSyncState = comment.syncState.description
                let commentData = comment.data
                let commentId = comment.commentId
                let commentUserId = comment.userId
                
                let dataToPrint = """
                Sync State: \(commentSyncState)

                ...

                Comment Data: \(commentData ?? [:])
                Comment Id: \(commentId)
                Comment User Id: \(commentUserId)
                """
                
                self.serverOutput = dataToPrint
            } catch {}
        }
    }
}

extension AmityCommentReferenceType {
    
    var identifier: String {
        switch self {
        case .post:
            return "Post"
        case .content:
            return "Content"
        default:
            return ""
        }
    }
}
